@@ -41,12 +41,13 @@
 #include "gedit-commands.h"
 #include "gedit-debug.h"
 #include "gedit-statusbar.h"
+#include "gedit-view-frame.h"
 #include "gedit-window.h"
 #include "gedit-window-private.h"
 #include "gedit-utils.h"
-#include "dialogs/gedit-search-dialog.h"
+#include "dialogs/gedit-replace-dialog.h"
 
-#define GEDIT_SEARCH_DIALOG_KEY		"gedit-search-dialog-key"
+#define GEDIT_REPLACE_DIALOG_KEY	"gedit-replace-dialog-key"
 #define GEDIT_LAST_SEARCH_DATA_KEY	"gedit-last-search-data-key"
 
 typedef struct _LastSearchData LastSearchData;
@@ -63,7 +64,7 @@ last_search_data_free (LastSearchData *data)
 }
 
 static void
-last_search_data_restore_position (GeditSearchDialog *dlg)
+last_search_data_restore_position (GeditReplaceDialog *dlg)
 {
 	LastSearchData *data;
 
@@ -78,7 +79,7 @@ last_search_data_restore_position (GeditSearchDialog *dlg)
 }
 
 static void
-last_search_data_store_position (GeditSearchDialog *dlg)
+last_search_data_store_position (GeditReplaceDialog *dlg)
 {
 	LastSearchData *data;
 
@@ -223,8 +224,8 @@ run_search (GeditView   *view,
 }
 
 static void
-do_find (GeditSearchDialog *dialog,
-	 GeditWindow       *window)
+do_find (GeditReplaceDialog *dialog,
+	 GeditWindow        *window)
 {
 	GeditView *active_view;
 	GeditDocument *doc;
@@ -246,12 +247,12 @@ do_find (GeditSearchDialog *dialog,
 
 	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (active_view)));
 
-	entry_text = gedit_search_dialog_get_search_text (dialog);
+	entry_text = gedit_replace_dialog_get_search_text (dialog);
 
-	match_case = gedit_search_dialog_get_match_case (dialog);
-	entire_word = gedit_search_dialog_get_entire_word (dialog);
-	search_backwards = gedit_search_dialog_get_backwards (dialog);
-	wrap_around = gedit_search_dialog_get_wrap_around (dialog);
+	match_case = gedit_replace_dialog_get_match_case (dialog);
+	entire_word = gedit_replace_dialog_get_entire_word (dialog);
+	search_backwards = gedit_replace_dialog_get_backwards (dialog);
+	wrap_around = gedit_replace_dialog_get_wrap_around (dialog);
 
 	GEDIT_SEARCH_SET_CASE_SENSITIVE (flags, match_case);
 	GEDIT_SEARCH_SET_ENTIRE_WORD (flags, entire_word);
@@ -277,7 +278,7 @@ do_find (GeditSearchDialog *dialog,
 		text_not_found (window, entry_text);
 
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
-					   GEDIT_SEARCH_DIALOG_REPLACE_RESPONSE,
+					   GEDIT_REPLACE_DIALOG_REPLACE_RESPONSE,
 					   found);
 }
 
@@ -308,6 +309,48 @@ get_selected_text (GtkTextBuffer  *doc,
 	return TRUE;
 }
 
+/* n: len of the string in bytes */
+static gboolean
+utf8_caselessnmatch (const char *s1,
+		     const char *s2,
+		     gssize      n1,
+		     gssize      n2)
+{
+	gchar *casefold;
+	gchar *normalized_s1;
+	gchar *normalized_s2;
+	gint len_s1;
+	gint len_s2;
+	gboolean ret = FALSE;
+
+	g_return_val_if_fail (s1 != NULL, FALSE);
+	g_return_val_if_fail (s2 != NULL, FALSE);
+	g_return_val_if_fail (n1 > 0, FALSE);
+	g_return_val_if_fail (n2 > 0, FALSE);
+
+	casefold = g_utf8_casefold (s1, n1);
+	normalized_s1 = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+	g_free (casefold);
+
+	casefold = g_utf8_casefold (s2, n2);
+	normalized_s2 = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+	g_free (casefold);
+
+	len_s1 = strlen (normalized_s1);
+	len_s2 = strlen (normalized_s2);
+
+	if (len_s1 < len_s2)
+		goto finally_2;
+
+	ret = (strncmp (normalized_s1, normalized_s2, len_s2) == 0);
+
+finally_2:
+	g_free (normalized_s1);
+	g_free (normalized_s2);
+
+	return ret;
+}
+
 static void
 replace_selected_text (GtkTextBuffer *buffer,
 		       const gchar   *replace)
@@ -325,8 +368,8 @@ replace_selected_text (GtkTextBuffer *buffer,
 }
 
 static void
-do_replace (GeditSearchDialog *dialog,
-	    GeditWindow       *window)
+do_replace (GeditReplaceDialog *dialog,
+	    GeditWindow        *window)
 {
 	GeditDocument *doc;
 	const gchar *search_entry_text;
@@ -340,12 +383,12 @@ do_replace (GeditSearchDialog *dialog,
 	if (doc == NULL)
 		return;
 
-	search_entry_text = gedit_search_dialog_get_search_text (dialog);
+	search_entry_text = gedit_replace_dialog_get_search_text (dialog);
 	g_return_if_fail ((search_entry_text) != NULL);
 	g_return_if_fail ((*search_entry_text) != '\0');
 
 	/* replace text may be "", we just delete */
-	replace_entry_text = gedit_search_dialog_get_replace_text (dialog);
+	replace_entry_text = gedit_replace_dialog_get_replace_text (dialog);
 	g_return_if_fail ((replace_entry_text) != NULL);
 
 	unescaped_search_text = gedit_utils_unescape_search_text (search_entry_text);
@@ -354,18 +397,18 @@ do_replace (GeditSearchDialog *dialog,
 			   &selected_text, 
 			   NULL);
 
-	match_case = gedit_search_dialog_get_match_case (dialog);
+	match_case = gedit_replace_dialog_get_match_case (dialog);
 
 	if ((selected_text == NULL) ||
-	    (match_case && (strcmp (selected_text, unescaped_search_text) != 0)) || 
-	    (!match_case && !g_utf8_caselessnmatch (selected_text,
-						    unescaped_search_text, 
-						    strlen (selected_text), 
-						    strlen (unescaped_search_text)) != 0))
+	    (match_case && (strcmp (selected_text, unescaped_search_text) != 0)) ||
+	    (!match_case && !utf8_caselessnmatch (selected_text,
+						  unescaped_search_text,
+						  strlen (selected_text),
+						  strlen (unescaped_search_text)) != 0))
 	{
 		do_find (dialog, window);
 		g_free (unescaped_search_text);
-		g_free (selected_text);	
+		g_free (selected_text);
 
 		return;
 	}
@@ -376,13 +419,13 @@ do_replace (GeditSearchDialog *dialog,
 	g_free (unescaped_search_text);
 	g_free (selected_text);
 	g_free (unescaped_replace_text);
-	
+
 	do_find (dialog, window);
 }
 
 static void
-do_replace_all (GeditSearchDialog *dialog,
-		GeditWindow       *window)
+do_replace_all (GeditReplaceDialog *dialog,
+		GeditWindow        *window)
 {
 	GeditView *active_view;
 	GeditDocument *doc;
@@ -399,16 +442,16 @@ do_replace_all (GeditSearchDialog *dialog,
 
 	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (active_view)));
 
-	search_entry_text = gedit_search_dialog_get_search_text (dialog);
+	search_entry_text = gedit_replace_dialog_get_search_text (dialog);
 	g_return_if_fail ((search_entry_text) != NULL);
 	g_return_if_fail ((*search_entry_text) != '\0');
 
 	/* replace text may be "", we just delete all occurrencies */
-	replace_entry_text = gedit_search_dialog_get_replace_text (dialog);
+	replace_entry_text = gedit_replace_dialog_get_replace_text (dialog);
 	g_return_if_fail ((replace_entry_text) != NULL);
 
-	match_case = gedit_search_dialog_get_match_case (dialog);
-	entire_word = gedit_search_dialog_get_entire_word (dialog);
+	match_case = gedit_replace_dialog_get_match_case (dialog);
+	entire_word = gedit_replace_dialog_get_entire_word (dialog);
 
 	GEDIT_SEARCH_SET_CASE_SENSITIVE (flags, match_case);
 	GEDIT_SEARCH_SET_ENTIRE_WORD (flags, entire_word);
@@ -428,26 +471,26 @@ do_replace_all (GeditSearchDialog *dialog,
 	}
 
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
-					   GEDIT_SEARCH_DIALOG_REPLACE_RESPONSE,
+					   GEDIT_REPLACE_DIALOG_REPLACE_RESPONSE,
 					   FALSE);
 }
 
 static void
-search_dialog_response_cb (GeditSearchDialog *dialog,
-			   gint               response_id,
-			   GeditWindow       *window)
+replace_dialog_response_cb (GeditReplaceDialog *dialog,
+			    gint                response_id,
+			    GeditWindow        *window)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
 	switch (response_id)
 	{
-		case GEDIT_SEARCH_DIALOG_FIND_RESPONSE:
+		case GEDIT_REPLACE_DIALOG_FIND_RESPONSE:
 			do_find (dialog, window);
 			break;
-		case GEDIT_SEARCH_DIALOG_REPLACE_RESPONSE:
+		case GEDIT_REPLACE_DIALOG_REPLACE_RESPONSE:
 			do_replace (dialog, window);
 			break;
-		case GEDIT_SEARCH_DIALOG_REPLACE_ALL_RESPONSE:
+		case GEDIT_REPLACE_DIALOG_REPLACE_ALL_RESPONSE:
 			do_replace_all (dialog, window);
 			break;
 		default:
@@ -457,9 +500,9 @@ search_dialog_response_cb (GeditSearchDialog *dialog,
 }
 
 static gboolean
-search_dialog_delete_event_cb (GtkWidget   *widget,
-			       GdkEventAny *event,
-			       gpointer     user_data)
+replace_dialog_delete_event_cb (GtkWidget   *widget,
+			        GdkEventAny *event,
+			        gpointer     user_data)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -468,13 +511,13 @@ search_dialog_delete_event_cb (GtkWidget   *widget,
 }
 
 static void
-search_dialog_destroyed (GeditWindow       *window,
-			 GeditSearchDialog *dialog)
+replace_dialog_destroyed (GeditWindow        *window,
+			  GeditReplaceDialog *dialog)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
 	g_object_set_data (G_OBJECT (window),
-			   GEDIT_SEARCH_DIALOG_KEY,
+			   GEDIT_REPLACE_DIALOG_KEY,
 			   NULL);
 	g_object_set_data (G_OBJECT (dialog),
 			   GEDIT_LAST_SEARCH_DATA_KEY,
@@ -482,28 +525,27 @@ search_dialog_destroyed (GeditWindow       *window,
 }
 
 static GtkWidget *
-create_dialog (GeditWindow *window,
-	       gboolean     show_replace)
+create_dialog (GeditWindow *window)
 {
 	GtkWidget *dialog;
 
-	dialog = gedit_search_dialog_new (GTK_WINDOW (window), show_replace);
+	dialog = gedit_replace_dialog_new (GTK_WINDOW (window));
 
 	g_signal_connect (dialog,
 			  "response",
-			  G_CALLBACK (search_dialog_response_cb),
+			  G_CALLBACK (replace_dialog_response_cb),
 			  window);
 	g_signal_connect (dialog,
 			 "delete-event",
-			 G_CALLBACK (search_dialog_delete_event_cb),
+			 G_CALLBACK (replace_dialog_delete_event_cb),
 			 NULL);
 
 	g_object_set_data (G_OBJECT (window),
-			   GEDIT_SEARCH_DIALOG_KEY,
+			   GEDIT_REPLACE_DIALOG_KEY,
 			   dialog);
 
 	g_object_weak_ref (G_OBJECT (dialog),
-			   (GWeakNotify) search_dialog_destroyed,
+			   (GWeakNotify) replace_dialog_destroyed,
 			   window);
 
 	return dialog;
@@ -513,57 +555,24 @@ void
 _gedit_cmd_search_find (GtkAction   *action,
 			GeditWindow *window)
 {
-	gpointer data;
-	GtkWidget *search_dialog;
-	GeditDocument *doc;
-	gboolean selection_exists;
-	gchar *find_text = NULL;
-	gint sel_len;
+	GeditView *active_view;
 
 	gedit_debug (DEBUG_COMMANDS);
 
-	data = g_object_get_data (G_OBJECT (window), GEDIT_SEARCH_DIALOG_KEY);
+	active_view = gedit_window_get_active_view (window);
+	if (active_view == NULL)
+		return;
 
-	if (data == NULL)
-	{
-		search_dialog = create_dialog (window, FALSE);
-	}
-	else
-	{
-		g_return_if_fail (GEDIT_IS_SEARCH_DIALOG (data));
-		
-		search_dialog = GTK_WIDGET (data);
-		
-		/* turn the dialog into a find dialog if needed */
-		if (gedit_search_dialog_get_show_replace (GEDIT_SEARCH_DIALOG (search_dialog)))
-		{
-			gedit_search_dialog_set_show_replace (GEDIT_SEARCH_DIALOG (search_dialog),
-							      FALSE);
-		}
-	}
-
-	doc = gedit_window_get_active_document (window);
-	g_return_if_fail (doc != NULL);
-
-	selection_exists = get_selected_text (GTK_TEXT_BUFFER (doc),
-					      &find_text,
-					      &sel_len);
-
-	if (selection_exists && find_text != NULL && sel_len < 80)
-	{
-		gedit_search_dialog_set_search_text (GEDIT_SEARCH_DIALOG (search_dialog),
-						     find_text);
-		g_free (find_text);
-	}
-	else
-	{
-		g_free (find_text);
-	}
-
-	gtk_widget_show (search_dialog);
-	last_search_data_restore_position (GEDIT_SEARCH_DIALOG (search_dialog));
-	gedit_search_dialog_present_with_time (GEDIT_SEARCH_DIALOG (search_dialog),
-					       GDK_CURRENT_TIME);
+	/* Focus the view if needed: we need to focus the view otherwise 
+	   activating the binding for goto line has no effect */
+	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	
+	/* incremental search is builtin in GeditView, just activate
+	 * the corresponding binding.
+	 */
+	gtk_bindings_activate (G_OBJECT (active_view),
+			       GDK_KEY_f,
+			       GDK_CONTROL_MASK);
 }
 
 void
@@ -579,24 +588,17 @@ _gedit_cmd_search_replace (GtkAction   *action,
 
 	gedit_debug (DEBUG_COMMANDS);
 
-	data = g_object_get_data (G_OBJECT (window), GEDIT_SEARCH_DIALOG_KEY);
+	data = g_object_get_data (G_OBJECT (window), GEDIT_REPLACE_DIALOG_KEY);
 
 	if (data == NULL)
 	{
-		replace_dialog = create_dialog (window, TRUE);
+		replace_dialog = create_dialog (window);
 	}
 	else
 	{
-		g_return_if_fail (GEDIT_IS_SEARCH_DIALOG (data));
+		g_return_if_fail (GEDIT_IS_REPLACE_DIALOG (data));
 		
 		replace_dialog = GTK_WIDGET (data);
-		
-		/* turn the dialog into a find dialog if needed */
-		if (!gedit_search_dialog_get_show_replace (GEDIT_SEARCH_DIALOG (replace_dialog)))
-		{
-			gedit_search_dialog_set_show_replace (GEDIT_SEARCH_DIALOG (replace_dialog),
-							      TRUE);
-		}
 	}
 
 	doc = gedit_window_get_active_document (window);
@@ -608,8 +610,8 @@ _gedit_cmd_search_replace (GtkAction   *action,
 
 	if (selection_exists && find_text != NULL && sel_len < 80)
 	{
-		gedit_search_dialog_set_search_text (GEDIT_SEARCH_DIALOG (replace_dialog),
-						     find_text);
+		gedit_replace_dialog_set_search_text (GEDIT_REPLACE_DIALOG (replace_dialog),
+						      find_text);
 		g_free (find_text);
 	}
 	else
@@ -618,9 +620,9 @@ _gedit_cmd_search_replace (GtkAction   *action,
 	}
 
 	gtk_widget_show (replace_dialog);
-	last_search_data_restore_position (GEDIT_SEARCH_DIALOG (replace_dialog));
-	gedit_search_dialog_present_with_time (GEDIT_SEARCH_DIALOG (replace_dialog),
-					       GDK_CURRENT_TIME);
+	last_search_data_restore_position (GEDIT_REPLACE_DIALOG (replace_dialog));
+	gedit_replace_dialog_present_with_time (GEDIT_REPLACE_DIALOG (replace_dialog),
+					        GDK_CURRENT_TIME);
 }
 
 static void
@@ -634,10 +636,10 @@ do_find_again (GeditWindow *window,
 	active_view = gedit_window_get_active_view (window);
 	g_return_if_fail (active_view != NULL);
 
-	data = g_object_get_data (G_OBJECT (window), GEDIT_SEARCH_DIALOG_KEY);
+	data = g_object_get_data (G_OBJECT (window), GEDIT_REPLACE_DIALOG_KEY);
 	
 	if (data != NULL)
-		wrap_around = gedit_search_dialog_get_wrap_around (GEDIT_SEARCH_DIALOG (data));
+		wrap_around = gedit_replace_dialog_get_wrap_around (GEDIT_REPLACE_DIALOG (data));
 	
 	run_search (active_view,
 		    wrap_around,
@@ -692,37 +694,12 @@ _gedit_cmd_search_goto_line (GtkAction   *action,
 	   activating the binding for goto line has no effect */
 	gtk_widget_grab_focus (GTK_WIDGET (active_view));
 
-
 	/* goto line is builtin in GeditView, just activate
-	 * the corrisponding binding.
+	 * the corresponding binding.
 	 */
-	gtk_bindings_activate (GTK_OBJECT (active_view),
-			       GDK_i,
+	gtk_bindings_activate (G_OBJECT (active_view),
+			       GDK_KEY_i,
 			       GDK_CONTROL_MASK);
 }
 
-void
-_gedit_cmd_search_incremental_search (GtkAction   *action,
-				      GeditWindow *window)
-{
-	GeditView *active_view;
-
-	gedit_debug (DEBUG_COMMANDS);
-
-	active_view = gedit_window_get_active_view (window);
-	if (active_view == NULL)
-		return;
-
-	/* Focus the view if needed: we need to focus the view otherwise 
-	   activating the binding for incremental search has no effect */
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
-	
-	/* incremental search is builtin in GeditView, just activate
-	 * the corrisponding binding.
-	 */
-	gtk_bindings_activate (GTK_OBJECT (active_view),
-			       GDK_k,
-			       GDK_CONTROL_MASK);
-}
-
-/* ex:ts=8:noet: */
+/* ex:set ts=8 noet: */
