@@ -243,12 +243,12 @@ gedit_panel_class_init (GeditPanelClass *klass)
 	binding_set = gtk_binding_set_by_class (klass);
 
 	gtk_binding_entry_add_signal (binding_set, 
-				      GDK_Escape, 
+				      GDK_KEY_Escape, 
 				      0, 
 				      "close", 
 				      0);
 	gtk_binding_entry_add_signal (binding_set, 
-				      GDK_Return, 
+				      GDK_KEY_Return, 
 				      GDK_CONTROL_MASK, 
 				      "focus_document", 
 				      0);
@@ -268,24 +268,6 @@ set_gtk_image_from_gtk_image (GtkImage *image,
 		case GTK_IMAGE_EMPTY:
 			gtk_image_clear (image);
 			break;
-		case GTK_IMAGE_PIXMAP:
-			{
-				GdkPixmap *pm;
-				GdkBitmap *bm;
-
-				gtk_image_get_pixmap (source, &pm, &bm);
-				gtk_image_set_from_pixmap (image, pm, bm);
-			}
-			break;
-		case GTK_IMAGE_IMAGE:
-			{
-				GdkImage *i;
-				GdkBitmap *bm;
-
-				gtk_image_get_image (source, &i, &bm);
-				gtk_image_set_from_image (image, i, bm);
-			}
-			break;
 		case GTK_IMAGE_PIXBUF:
 			{
 				GdkPixbuf *pb;
@@ -987,4 +969,4 @@ _gedit_panel_set_active_item_by_id (GeditPanel *panel,
 	}
 }
 
-/* ex:ts=8:noet: */
+/* ex:set ts=8 noet: */
