@@ -24,7 +24,8 @@
 #include "gedit-marshal.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
-#include "gedit-overlay.h"
+#include "gedit-animated-overlay.h"
+#include "gedit-floating-slider.h"
 #include "gedit-rounded-frame.h"
 
 #include <gdk/gdkkeysyms.h>
@@ -66,12 +67,14 @@ struct _GeditViewFramePrivate
 	 */
 	guint        search_flags;
 
+	GtkWidget   *slider;
 	GtkWidget   *search_widget;
 	GtkWidget   *search_entry;
 
 	guint        typeselect_flush_timeout;
 	glong        view_scroll_event_id;
 	glong        search_entry_focus_out_id;
+	glong        search_entry_changed_id;
 
 	guint        disable_popdown : 1;
 	guint        wrap_around : 1;
@@ -148,13 +151,15 @@ hide_search_widget (GeditViewFrame *frame,
                     gboolean        cancel)
 {
 	GtkTextBuffer *buffer;
-	GdkGravity gravity;
 
 	if (frame->priv->disable_popdown)
 	{
 		return;
 	}
 
+	g_signal_handler_block (frame->priv->search_entry,
+	                        frame->priv->search_entry_focus_out_id);
+
 	if (frame->priv->view_scroll_event_id != 0)
 	{
 		g_signal_handler_disconnect (frame->priv->view,
@@ -162,38 +167,16 @@ hide_search_widget (GeditViewFrame *frame,
 		frame->priv->view_scroll_event_id = 0;
 	}
 
-	if (frame->priv->search_entry_focus_out_id != 0)
-	{
-		g_signal_handler_disconnect (frame->priv->search_entry,
-		                             frame->priv->search_entry_focus_out_id);
-		frame->priv->search_entry_focus_out_id = 0;
-	}
-
 	if (frame->priv->typeselect_flush_timeout != 0)
 	{
 		g_source_remove (frame->priv->typeselect_flush_timeout);
 		frame->priv->typeselect_flush_timeout = 0;
 	}
 
-	if (gtk_widget_get_direction (frame->priv->search_entry) == GTK_TEXT_DIR_RTL)
-	{
-		gravity = GDK_GRAVITY_NORTH_WEST;
-	}
-	else
-	{
-		gravity = GDK_GRAVITY_NORTH_EAST;
-	}
-
-	gedit_overlay_slide (GEDIT_OVERLAY (frame->priv->overlay),
-	                     frame->priv->search_widget,
-	                     300,
-	                     GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN_OUT,
-	                     GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_UPSTAGE,
-	                     GTK_ORIENTATION_VERTICAL,
-	                     gravity,
-	                     SEARCH_POPUP_OFFSET,
-	                     FALSE);
-	frame->priv->search_widget = NULL;
+	/* To hide the slider we just set the animation-state property */
+	g_object_set (G_OBJECT (frame->priv->slider),
+	              "animation-state", GEDIT_THEATRICS_ANIMATION_STATE_INTENDING_TO_GO,
+	              NULL);
 
 	if (cancel)
 	{
@@ -214,6 +197,9 @@ hide_search_widget (GeditViewFrame *frame,
 	/* Make sure the view is the one who has the focus when we destroy
 	   the search widget */
 	gtk_widget_grab_focus (frame->priv->view);
+
+	g_signal_handler_unblock (frame->priv->search_entry,
+	                          frame->priv->search_entry_focus_out_id);
 }
 
 static void
@@ -1078,11 +1064,6 @@ create_search_widget (GeditViewFrame *frame)
 	g_signal_connect (search_widget, "scroll-event",
 	                  G_CALLBACK (search_widget_scroll_event),
 	                  frame);
-	/* Manage the scroll also for the view */
-	frame->priv->view_scroll_event_id =
-		g_signal_connect (frame->priv->view, "scroll-event",
-			          G_CALLBACK (search_widget_scroll_event),
-			          frame);
 
 	hbox = gtk_hbox_new (FALSE, 3);
 	gtk_widget_show (hbox);
@@ -1107,9 +1088,10 @@ create_search_widget (GeditViewFrame *frame)
 	g_signal_connect (frame->priv->search_entry, "insert_text",
 	                  G_CALLBACK (search_entry_insert_text),
 	                  frame);
-	g_signal_connect (frame->priv->search_entry, "changed",
-	                  G_CALLBACK (search_init),
-	                  frame);
+	frame->priv->search_entry_changed_id =
+		g_signal_connect (frame->priv->search_entry, "changed",
+		                  G_CALLBACK (search_init),
+		                  frame);
 	frame->priv->search_entry_focus_out_id =
 		g_signal_connect (frame->priv->search_entry, "focus-out-event",
 		                  G_CALLBACK (search_entry_focus_out_event),
@@ -1205,6 +1187,8 @@ init_search_entry (GeditViewFrame *frame)
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (frame->priv->view));
 
+	customize_for_search_mode (frame);
+
 	if (frame->priv->search_mode == GOTO_LINE)
 	{
 		gint   line;
@@ -1243,6 +1227,19 @@ init_search_entry (GeditViewFrame *frame)
 		{
 			frame->priv->old_search_text = old_find_text;
 			add_search_completion_entry (old_find_text);
+			g_signal_handler_block (frame->priv->search_entry,
+			                        frame->priv->search_entry_changed_id);
+
+			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
+			                    old_find_text);
+
+			g_signal_handler_unblock (frame->priv->search_entry,
+			                          frame->priv->search_entry_changed_id);
+		}
+		else
+		{
+			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
+			                    "");
 		}
 
 		if (old_find_flags != 0)
@@ -1259,11 +1256,6 @@ init_search_entry (GeditViewFrame *frame)
 			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
 			                    find_text);
 		}
-		else
-		{
-			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
-			                    "");
-		}
 
 		g_free (find_text);
 	}
@@ -1275,12 +1267,10 @@ start_interactive_search_real (GeditViewFrame *frame)
 	GtkTextBuffer *buffer;
 	GtkTextMark *mark;
 	GtkTextIter iter;
-	GdkGravity gravity;
 
 	/* FIXME: it enters here twice, why? */
 
-	if ((frame->priv->search_widget != NULL) &&
-	    gtk_widget_get_visible (frame->priv->search_widget))
+	if (gtk_widget_get_visible (frame->priv->slider))
 	{
 		if (frame->priv->search_mode != frame->priv->request_search_mode)
 		{
@@ -1309,29 +1299,19 @@ start_interactive_search_real (GeditViewFrame *frame)
 	frame->priv->start_mark = gtk_text_buffer_create_mark (buffer, NULL,
 	                                                       &iter, FALSE);
 
-	frame->priv->search_widget = create_search_widget (frame);
-
-	if (gtk_widget_get_direction (frame->priv->search_entry) == GTK_TEXT_DIR_RTL)
-	{
-		gravity = GDK_GRAVITY_NORTH_WEST;
-	}
-	else
-	{
-		gravity = GDK_GRAVITY_NORTH_EAST;
-	}
-
-	gedit_overlay_slide (GEDIT_OVERLAY (frame->priv->overlay),
-	                     frame->priv->search_widget,
-	                     300,
-	                     GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN_OUT,
-	                     GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_DOWNSTAGE,
-	                     GTK_ORIENTATION_VERTICAL,
-	                     gravity,
-	                     SEARCH_POPUP_OFFSET,
-	                     TRUE);
+	/* To slide in we set the right animation state in the animatable */
+	g_object_set (G_OBJECT (frame->priv->slider),
+	              "animation-state", GEDIT_THEATRICS_ANIMATION_STATE_COMING,
+	              NULL);
 
 	init_search_entry (frame);
 
+	/* Manage the scroll also for the view */
+	frame->priv->view_scroll_event_id =
+		g_signal_connect (frame->priv->view, "scroll-event",
+			          G_CALLBACK (search_widget_scroll_event),
+			          frame);
+
 	frame->priv->typeselect_flush_timeout =
 		g_timeout_add (GEDIT_VIEW_FRAME_SEARCH_DIALOG_TIMEOUT,
 		               (GSourceFunc) search_entry_flush_timeout,
@@ -1406,6 +1386,7 @@ gedit_view_frame_init (GeditViewFrame *frame)
 {
 	GeditDocument *doc;
 	GtkWidget *sw;
+	GeditOverlayChildPosition position;
 
 	frame->priv = GEDIT_VIEW_FRAME_GET_PRIVATE (frame);
 
@@ -1433,13 +1414,10 @@ gedit_view_frame_init (GeditViewFrame *frame)
 	                  G_CALLBACK (on_start_interactive_goto_line),
 	                  frame);
 
-	frame->priv->overlay = gedit_overlay_new (frame->priv->view);
-	gtk_widget_show (frame->priv->overlay);
-
 	/* Create the scrolled window */
 	sw = gtk_scrolled_window_new (NULL, NULL);
 
-	gtk_container_add (GTK_CONTAINER (sw), frame->priv->overlay);
+	gtk_container_add (GTK_CONTAINER (sw), frame->priv->view);
 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 					GTK_POLICY_AUTOMATIC,
@@ -1449,7 +1427,33 @@ gedit_view_frame_init (GeditViewFrame *frame)
 					     GTK_SHADOW_IN);
 	gtk_widget_show (sw);
 
-	gtk_box_pack_start (GTK_BOX (frame), sw, TRUE, TRUE, 0);
+	frame->priv->overlay = gedit_animated_overlay_new (sw, frame->priv->view);
+	gtk_widget_show (frame->priv->overlay);
+
+	gtk_box_pack_start (GTK_BOX (frame), frame->priv->overlay, TRUE, TRUE, 0);
+
+	/* Add slider */
+	if (gtk_widget_get_direction (GTK_WIDGET (frame)) == GTK_TEXT_DIR_RTL)
+	{
+		position = GEDIT_OVERLAY_CHILD_POSITION_NORTH_WEST;
+	}
+	else
+	{
+		position = GEDIT_OVERLAY_CHILD_POSITION_NORTH_EAST;
+	}
+
+	frame->priv->search_widget = create_search_widget (frame);
+	frame->priv->slider = gedit_floating_slider_new (frame->priv->search_widget);
+	g_object_set (G_OBJECT (frame->priv->slider),
+	              "position", position,
+	              "offset", SEARCH_POPUP_OFFSET,
+	              "easing", GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN_OUT,
+	              "blocking", GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_DOWNSTAGE,
+	              "orientation", GTK_ORIENTATION_VERTICAL,
+	              NULL);
+
+	gedit_animated_overlay_add (GEDIT_ANIMATED_OVERLAY (frame->priv->overlay),
+	                            GEDIT_ANIMATABLE (frame->priv->slider));
 }
 
 GeditViewFrame *
